﻿using System;
using System.IO;
using System.Xml;
using UnityEditor;
using UnityEngine;
using FYBase;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using fybase.yx;
using System.Collections;

namespace fybase
{
    internal enum GameOrientation
    {
        Landscape = 0,
        Portrait = 1
    }
    internal enum IssueArea
    {
        Inland = 1,
        Oversea = 2
    }
    internal struct FYCfgInfo
    {
        public const string XcodeConfigPath = "Assets/Plugins/FYBase/Editor/XCodeConfig.json";
        public string xml;
        public string plist;
        public string javaActivity;
        public string javaAppC;
        public string javaBridge;
        public string objcAppC;
        public string objcBridge;
        public string xcodeConfig;
    }
    internal struct FYFileTree
    {
        public string sdkName;
        public string version;
        public List<string> rootDir;
        public List<string> files;
        public FYFileTree(string sdkName, string version, List<string> rootDir, List<string> files)
        {
            this.sdkName = sdkName;
            this.version = version;
            this.rootDir = rootDir;
            this.files = files;
        }
    }
    public class ConfigWindow : EditorWindow
    {
        [InitializeOnLoadMethod]//编译完成时执行
        static void RegisterConfig()
        {
            //注册Android配置事件，配置时会添加该路径的配置,优先级最高
            AddAndroidConfigPathsAct += dic => dic.Add("Assets/Plugins/FYBase/Editor/AndroidBuildConfig.json", 0);
            //注册iOS配置事件，配置时会添加该路径的配置,优先级最高
            AddiOSConfigPathsAct += dic => dic.Add("Assets/Plugins/FYBase/Editor/iOSBuildConfig.json", 0);

            DoDeleteExtraFilesAct += () => DoDelete(FYBASE_TREE);
        }
        /// <summary>
        /// 融合Android配置事件，string为配置文件路径，int为优先级，值越小优先级越高,建议大于0开始，第三方插件添加配置文件(格式参考<seealso href="Assets/Plugins/FYBase/Editor/AndroidBuildConfig.json">AndroidBuildConfig.json</seealso>)到Android的Java或AndroidManifest文件中,具体用例可参考<see cref="ConfigWindow.RegisterConfig"/>
        /// </summary>
        public static event Action<Dictionary<string, int>> AddAndroidConfigPathsAct;
        /// <summary>
        /// 融合iOS配置事件，string为配置文件路径，int为优先级，值越小优先级越高,建议大于0开始，第三方插件添加配置文件(格式参考<seealso href="Assets/Plugins/FYBase/Editor/iOSBuildConfig.json">iOSBuildConfig.json</seealso>)到Xcode的objc或Plist文件中,具体用例可参考<see cref="ConfigWindow.RegisterConfig"/>
        /// </summary>
        public static event Action<Dictionary<string, int>> AddiOSConfigPathsAct;

        internal static event Action DoDeleteExtraFilesAct;
        internal const string FYBASE_TREE = "Assets/Plugins/FYBase/Editor/fybase.fytree";

        internal const string androidPath = "Assets/Plugins/Android/";

        internal const string desXMLPath = "Assets/Plugins/Android/AndroidManifest.xml";
        internal const string desJavaPath = "Assets/Plugins/FYBase/Android/FYSDKBridge";

        internal const string desPlistPath = "Assets/Plugins/FYBase/iOS/FYCoreConfig.plist";
        internal const string desObjcPath = "Assets/Plugins/FYBase/iOS/FYSDKBridge";

        internal const string assetsForAD = "Assets/Plugins/Android/assets";
        internal const string desSupplierconfig = "Assets/Plugins/Android/assets/supplierconfig.json";
        void OnEnable()
        {
            InitConfig(out companyId, out appID, out appKey, out debug, out orientation, out issueArea);
        }

        internal static void InitConfig(out int companyId, out int appID, out string appKey, out bool debug, out GameOrientation orientation, out IssueArea gameIssue)
        {
            companyId = ORI_COMPANY_ID;
            appID = ORI_APPID;
            appKey = ORI_APP_KEY;
            debug = ORI_DEBUG;
            orientation = ORI_ORIENTATION;
            gameIssue = ORI_ISSUEAREA;
            if (!File.Exists(desXMLPath)) return;
            XmlDocument FYandroidManifest = new XmlDocument();
            FYandroidManifest.Load(desXMLPath);
            XmlElement FYmanifest = FYandroidManifest.DocumentElement;
            XmlNode FYapplication = FYmanifest.SelectSingleNode("application");
            if (FYapplication == null) return;
            Dictionary<string, string> FYMetas = new Dictionary<string, string>();
            XmlNodeList metas = FYapplication.SelectNodes("meta-data");
            foreach (XmlNode act in metas)
            {
                string key = act.Attributes.GetNamedItem("android:name").Value;
                if (FYMetas.ContainsKey(key))
                {
                    Debug.LogError($"FYINFO: 请检验{desXMLPath}文件中SDK基础配置的重复节点{key}，并将重复节点删除");
                }
                else
                {
                    FYMetas.Add(key, act.Attributes.GetNamedItem("android:value").Value);
                }
            }
            if (FYMetas.ContainsKey("FY_GAME_COMPANYID"))
                int.TryParse(FYMetas["FY_GAME_COMPANYID"], out companyId);
            if (FYMetas.ContainsKey("FY_GAME_APPID"))
                int.TryParse(FYMetas["FY_GAME_APPID"], out appID);
            if (FYMetas.ContainsKey("FY_GAME_APPKEY"))
            {
                if (FYMetas["FY_GAME_APPKEY"].Trim() != "")
                {
                    try
                    {
                        appKey = AESUtil.DecryptString(FYMetas["FY_GAME_APPKEY"]);
                    }
                    catch
                    {
                        Debug.LogError($"FYINFO: APPKEY错误，{HelpInfo}");
                    }
                }
            }
            if (FYMetas.ContainsKey("FY_GAME_ORIENTATION"))
            {
                int.TryParse(FYMetas["FY_GAME_ORIENTATION"], out int ori);
                orientation = (GameOrientation)ori;
            }
            if (FYMetas.ContainsKey("FY_GAME_DEBUG"))
            {
                int.TryParse(FYMetas["FY_GAME_DEBUG"], out int isd);
                debug = isd == 1;
            }

            if (FYMetas.ContainsKey("FY_GAME_EXTRA_JSON"))
            {
                Hashtable table = FYMetas["FY_GAME_EXTRA_JSON"].hashtableFromJson();
                if (table.ContainsKey("game_issued"))
                {
                    int.TryParse(table.SGet<string>("game_issued"), out int gi);
                    gameIssue = (IssueArea)gi;
                }
            }
        }
        //[MenuItem("Window/FYConfigTool")]
        [MenuItem("Window/星云配置")]
        internal static void ShowWindow()
        {
            EditorWindow.GetWindow(typeof(ConfigWindow), true, "星云配置", true);
        }
        internal const string HelpInfo = "请在菜单栏点击 \"Window => 星云配置\" 的配置界面配置SDK信息,配置完毕点击【确定】完成配置";
        internal const int ORI_COMPANY_ID = 0;
        internal const int ORI_APPID = 0;
        internal const string ORI_APP_KEY = "";
        internal const bool ORI_DEBUG = false;
        internal const GameOrientation ORI_ORIENTATION = GameOrientation.Landscape;
        internal const IssueArea ORI_ISSUEAREA = IssueArea.Inland;
        int companyId;
        int appID;
        string appKey;
        bool debug;
        GameOrientation orientation;
        IssueArea issueArea;

        //执行这一个函数来一个自定义检视面板
        void OnGUI()
        {
            //设置整个界面是以垂直方向来布局
            EditorGUILayout.BeginVertical();
            EditorGUILayout.Space();

            companyId = EditorGUILayout.IntField("Company ID", companyId);
            bool tp = false;
            if (companyId == ORI_COMPANY_ID)
            {
                tp = true;
                EditorGUILayout.HelpBox("请输入Company ID", MessageType.Error);
            }
            appID = EditorGUILayout.IntField("App ID", appID);
            if (appID == ORI_APPID)
            {
                tp = true;
                EditorGUILayout.HelpBox("请输入APP ID", MessageType.Error);
            }
            appKey = EditorGUILayout.TextField("App Key", appKey);
            if (appKey == ORI_APP_KEY)
            {
                tp = true;
                EditorGUILayout.HelpBox("请输入APP KEY", MessageType.Error);
            }
            debug = EditorGUILayout.Toggle("Debug", debug);
            orientation = (GameOrientation)EditorGUILayout.EnumPopup("Game Orientation", orientation);
            issueArea = (IssueArea)EditorGUILayout.EnumPopup("Issue Area", issueArea);
            EditorGUILayout.Space();

            if (tp)
            {
                EditorGUILayout.HelpBox("请先将配置信息填写完毕", MessageType.Error);
            }
            else
            {
                EditorGUILayout.HelpBox($"如果您的项目包含{desXMLPath},请先备份，以防覆盖原先配置信息", MessageType.Info);

                if (GUILayout.Button("确定"))
                {
                    FYCfgInfo info = Config(companyId.ToString(), appID.ToString(), appKey, debug, orientation, issueArea);
                    if (File.Exists(desXMLPath))
                    {
                        if (EditorUtility.DisplayDialog("星云配置", $"检测到{desXMLPath}已存在，将自动融合一些SDK配置信息", "确定", "取消"))
                        {
                            WriteToDest(info);
                        }
                        else
                        {
                            if (EditorUtility.DisplayDialog("取消配置", $"已取消配置，接入的星云SDK可能无法正常使用，{HelpInfo}", "确定"))
                            {
                                Debug.LogWarning($"FYINFO: 已取消配置，接入的星云SDK可能无法正常使用，{HelpInfo}");
                            }
                        }
                    }
                    else
                    {
                        WriteToDest(info);
                    }
                }
            }
            EditorGUILayout.EndVertical();
        }
        internal static FYCfgInfo Config(string companyId, string appid, string appkey, bool debug, GameOrientation orientation, IssueArea gameIssue)
        {
            int defaultOrientation = 0;
            switch (PlayerSettings.defaultInterfaceOrientation)
            {
                case UIOrientation.Portrait:
                case UIOrientation.PortraitUpsideDown:
                    defaultOrientation = 1;
                    break;
                case UIOrientation.LandscapeRight:
                case UIOrientation.LandscapeLeft:
                    defaultOrientation = 0;
                    break;
                case UIOrientation.AutoRotation:
                    if (PlayerSettings.allowedAutorotateToLandscapeLeft || PlayerSettings.allowedAutorotateToLandscapeRight)
                    {
                        defaultOrientation = 0;
                    }
                    else
                    {
                        defaultOrientation = 1;
                    }
                    break;
            }
            if ((int)orientation != defaultOrientation)
            {
                Debug.LogWarning($"FYINFO: 星云配置界面中你所选的屏幕朝向({orientation})和PlayerSettings设置的朝向({(GameOrientation)defaultOrientation})不一致，可能会影响一些SDK的界面展示，请考虑保持统一");
            }

            var AndroidConfigPathsDic = new Dictionary<string, int>();
            AddAndroidConfigPathsAct?.Invoke(AndroidConfigPathsDic);
            var AndroidConfigPaths = AndroidConfigPathsDic.OrderBy(kv => kv.Value).Select(kv => kv.Key);

            var iOSConfigPathsDic = new Dictionary<string, int>();
            AddiOSConfigPathsAct?.Invoke(iOSConfigPathsDic);
            var iOSConfigPaths = iOSConfigPathsDic.OrderBy(kv => kv.Value).Select(kv => kv.Key);

            var androidCfgs = new List<string>();
            var iOSCfgs = new List<string>();
            foreach (var path in AndroidConfigPaths)
            {
                if (File.Exists(path))
                    androidCfgs.Add(File.ReadAllText(path));
                else
                    Debug.LogError($"未找到Android配置文件{path}");
            }
            foreach (var path in iOSConfigPaths)
            {
                if (File.Exists(path))
                    iOSCfgs.Add(File.ReadAllText(path));
                else
                    Debug.LogError($"未找到iOS配置文件{path}");
            }
            CodeUtil.GenerateConfig(androidCfgs, out string xml, out string JApplication, out string JActivity, out string JBridge, iOSCfgs, out string xcode, out string plist, out string MAppController, out string MBridge, companyId, appid, appkey, debug, (int)orientation, (int)gameIssue);
            return new FYCfgInfo()
            {
                xml = xml,
                plist = plist,
                javaActivity = JActivity,
                javaAppC = JApplication,
                javaBridge = JBridge,
                objcAppC = MAppController,
                objcBridge = MBridge,
                xcodeConfig = xcode
            };
        }
        /// <summary>
        /// 将Editor下的文件复制到对应Plugins目录下，如果AndroidManifest.xml包含非FYSDK的信息，可能会覆盖掉~
        /// </summary>
        private static void WriteToDest(FYCfgInfo cfg)
        {
            DoDeleteExtraFilesAct?.Invoke();
            WriteAndroid(cfg);
            WriteiOS(cfg);
            AssetDatabase.Refresh();
            Debug.Log($"FYINFO: 星云配置完成");
        }
        private static void WriteAndroid(FYCfgInfo cfg)
        {
            if (!Directory.Exists(androidPath))
                Directory.CreateDirectory(androidPath);
            if (File.Exists(desXMLPath))
            {
                List<string> tips = new List<string>();
                XmlDocument androidManifest = MergeXml(cfg.xml, tips);
                androidManifest.Save(desXMLPath);
                Debug.LogWarning($"FYINFO: 检测到{desXMLPath}已存在，将自动融合一些配置~，以下为本次融合记录：\r\n{(tips.Count > 0 ? string.Join(string.Empty, tips) : "无记录")}");
            }
            else
            {
                File.WriteAllText(desXMLPath, cfg.xml);
            }
            if (File.Exists(desSupplierconfig))
            {
                File.Delete(desSupplierconfig);
                var mf = $"{desSupplierconfig}.meta";
                if (File.Exists(mf)) File.Delete(mf);
                Debug.LogWarning($"FYINFO: {desSupplierconfig}已迁移到AAR中，不再需要此文件");
            }
            if (Directory.Exists(assetsForAD)&&Directory.GetDirectories(assetsForAD).Length == 0 && Directory.GetFiles(assetsForAD).Length == 0)
            {
                Directory.Delete(assetsForAD);
                var mf = $"{assetsForAD}.meta";
                if (File.Exists(mf)) File.Delete(mf);
                Debug.LogWarning($"FYINFO: 删除{assetsForAD}空文件夹");
            }
            if (Directory.Exists(desJavaPath))
                Directory.Delete(desJavaPath, true);
            Directory.CreateDirectory(desJavaPath);
            File.WriteAllText($"{desJavaPath}/FYSDKActivity.java", cfg.javaActivity);
            File.WriteAllText($"{desJavaPath}/FYSDKApplication.java", cfg.javaAppC);
            File.WriteAllText($"{desJavaPath}/FYSDKBridge.java", cfg.javaBridge);
        }
        private static void WriteiOS(FYCfgInfo cfg)
        {
            //File.WriteAllText(FYCfgInfo.XcodeConfigPath, Regex.Unescape(cfg.xcodeConfig));//全部转义
            File.WriteAllText(FYCfgInfo.XcodeConfigPath, new Regex(@"\\u([0-9A-F]{4})", RegexOptions.IgnoreCase | RegexOptions.Compiled)
                .Replace(cfg.xcodeConfig, x => string.Empty + Convert.ToChar(Convert.ToUInt16(x.Result("$1"), 16))));//各国文字转换
            File.WriteAllText(desPlistPath, cfg.plist);
            if (Directory.Exists(desObjcPath))
                Directory.Delete(desObjcPath, true);
            Directory.CreateDirectory(desObjcPath);
            File.WriteAllText($"{desObjcPath}/FYSDKAppController.mm", cfg.objcAppC);
            File.WriteAllText($"{desObjcPath}/FYSDKBridge.mm", cfg.objcBridge);
        }
        internal static void DoDelete(string treePath)
        {
            if (!File.Exists(treePath))
                return;
            var tree = JsonUtility.FromJson<FYFileTree>(File.ReadAllText(treePath));
            HashSet<string> retainFiles = new HashSet<string>();
            foreach (var ph in tree.files)
            {
                retainFiles.Add(ph);
            }
            List<string> allFiles = new List<string>();
            foreach (var path in tree.rootDir)
            {
                allFiles.AddRange(Directory.GetFiles(path, "*", SearchOption.AllDirectories).Where(fn => !fn.EndsWith(".meta")).Select(fn => fn.Replace("\\", "/")));
            }
            IEnumerable<string> deleteFiles = allFiles.Where(fn => !retainFiles.Contains(fn));
            foreach (var delFile in deleteFiles)
            {
                File.Delete(delFile);
                if (File.Exists(delFile + ".meta"))
                    File.Delete(delFile + ".meta");
            }
            deleteFiles = deleteFiles.Where(fn => !fn.StartsWith(desJavaPath) && !fn.StartsWith(desObjcPath) && !fn.StartsWith(desPlistPath));
            if (deleteFiles.Count() > 0)
            {
                Debug.LogWarning($"FYINFO: 此次检测到<color=blue>{tree.sdkName}.unitypacakage(版本:{tree.version})</color>的多余文件，请检验以下文件删除后是否对您当前项目有影响(或删除原SDK的所有文件后重新导入新SDK的unitypacakage，再重新配置):\r\n{string.Join(string.Empty, deleteFiles.Select(s => $"<color=red>{s}</color>\r\n"))}");
            }
            //else
            //{
            //    Debug.Log($"未找到旧<color=green>{tree.sdkName}</color>的余留文件，无需清理");
            //}
        }
        #region 自动合并xml,如果融合冲突，请依照提示解决
        internal static XmlDocument MergeXml(string oriXml, List<string> tips)
        {
            XmlDocument FYandroidxml = new XmlDocument();
            FYandroidxml.LoadXml(oriXml);
            XmlElement FYmanifest = FYandroidxml.DocumentElement;

            XmlDocument androidxml = new XmlDocument();
            androidxml.Load(desXMLPath);
            XmlElement manifest = androidxml.DocumentElement;
            androidxml.CreateXmlDeclaration("1.0", "utf-8", null);

            SetAttr(manifest, "xmlns:android", FYmanifest);
            //SetAttr(manifest, "xmlns:tools", FYmanifest);
            //SetAttr(manifest, "android:installLocation", FYmanifest);

            //SetNode(manifest, "supports-screens", FYmanifest);

            // 合并 permission

            Dictionary<string, XmlNode> FYPermission = new Dictionary<string, XmlNode>();
            XmlNodeList permissions = FYmanifest.SelectNodes("uses-permission");

            foreach (XmlNode act in permissions)
            {
                XmlNode per = act.Attributes.GetNamedItem("android:name");
                if (!FYPermission.ContainsKey(per.Value))
                {
                    FYPermission.Add(per.Value, act);
                }
            }
            permissions = manifest.SelectNodes("uses-permission");
            XmlNode refNode = null;
            List<XmlNode> rmPermissions = new List<XmlNode>();
            foreach (XmlNode act in permissions)
            {
                XmlNode per = act.Attributes.GetNamedItem("android:name");
                refNode = act;
                if (FYPermission.ContainsKey(per.Value))
                    FYPermission.Remove(per.Value);
                else
                    rmPermissions.Add(act);
            }
            foreach (XmlNode act in FYPermission.Values)
            {
                XmlNode node_new = manifest.OwnerDocument.ImportNode(act, true);
                if (refNode == null)
                    manifest.AppendChild(node_new);
                else
                    manifest.InsertAfter(node_new, refNode);
                tips.Add($"<color=green>新增:<uses-permission android:name=\"{ node_new.Attributes.GetNamedItem("android:name").Value}\" /> </color>\r\n");
            }
            /*可能冗余的权限先不删除，因为有可能是其他SDK需要的权限
			foreach (var rmPer in rmPermissions) {
                manifest.RemoveChild(rmPer);
                if(!ignore)
                    tips.Add($"<color=red>删除:<uses-permission android:name=\"{ rmPer.Attributes.GetNamedItem("android:name").Value}\" /> </color> !!!若此权限归于其他SDK必备权限，请考虑是否重新添加!!!\r\n");
            }*/

            // 合并application

            XmlNode application = manifest.SelectSingleNode("application");
            XmlNode FYapplication = FYmanifest.SelectSingleNode("application");
            if (application == null)
            {
                XmlNode node_new = manifest.OwnerDocument.ImportNode(FYapplication, true);
                manifest.AppendChild(node_new);
                tips.Add($"<color=green>新增:application节点及其子节点 </color>\r\n");
            }
            else
            {
                XmlNode attr = application.Attributes.GetNamedItem("android:name");
                string val = FYapplication.Attributes.GetNamedItem("android:name").Value;
                if (attr != null && attr.Value != val)
                {
                    throw new ArgumentException($"application配置冲突，android:name的属性{attr.Value}与目标配置{val}不符，请使用以下方案调整：\r\n\r\n方案一:在Assets/Plugins目录下新建目录(XXXConfig/Editor/)，并创建编辑器脚本，参考{nameof(ConfigWindow)}.{nameof(RegisterConfig)}方法，新建AndroidBuildConfig.json并填写: {{\"FYSDKApplication\":{{\"superClass\":\"{attr.Value}\"}}}}，并注册自定义的SDK配置路径到事件{nameof(ConfigWindow)}.{nameof(AddAndroidConfigPathsAct)}中,最后将AndroidManifest文件的{attr.Value}修改为{val}\r\n\r\n方案二(每次星云配置后会还原修改，不推荐)：请考虑使用继承方式({val} extends {attr.Value})或者将{attr.Value}代码融合到{val}中,最后将AndroidManifest文件的{attr.Value}修改为{val}\r\n\r\n");
                }
                else
                {
                    if (!IsSatisfyAttrs(application, FYapplication))
                    {
                        SetAttr(application, "android:name", FYapplication);
                        SetAttr(application, "android:theme", FYapplication);
                        SetAttr(application, "android:label", FYapplication);
                        tips.Add($"<color=blue>替换:<application android:name=\"{val}\" android:theme=\"{FYapplication.Attributes.GetNamedItem("android:theme").Value}\" android:label=\"{FYapplication.Attributes.GetNamedItem("android:label").Value}\"> </color>\r\n");
                    }
                }
                Dictionary<string, XmlNode> FYMetas = new Dictionary<string, XmlNode>();
                XmlNodeList metas = FYapplication.SelectNodes("meta-data");
                foreach (XmlNode act in metas)
                {
                    XmlNode per = act.Attributes.GetNamedItem("android:name");
                    FYMetas.Add(per.Value, act);
                }
                metas = application.SelectNodes("meta-data");
                refNode = null;
                foreach (XmlNode act in metas)
                {
                    XmlNode per = act.Attributes.GetNamedItem("android:name");
                    refNode = refNode ?? act;
                    if (FYMetas.ContainsKey(per.Value))
                    {
                        refNode = act;
                        if (!IsSatisfyAttrs(act, FYMetas[per.Value]))
                        {
                            string mv = FYMetas[per.Value].Attributes.GetNamedItem("android:value").Value;
                            act.Attributes.GetNamedItem("android:value").Value = mv;
                            tips.Add($"<color=blue>替换:<meta-data android:name=\"{per.Value}\" android:value=\"{mv}\" /> </color>\r\n");
                        }
                        FYMetas.Remove(per.Value);
                    }
                }
                foreach (XmlNode act in FYMetas.Values)
                {
                    XmlNode node_new = manifest.OwnerDocument.ImportNode(act, true);
                    if (refNode == null)
                        application.AppendChild(node_new);
                    else
                        application.InsertBefore(node_new, refNode);
                    tips.Add($"<color=green>新增:<meta-data android:name=\"{node_new.Attributes.GetNamedItem("android:name").Value}\" android:value=\"{node_new.Attributes.GetNamedItem("android:value").Value}\" /> </color>\r\n");
                }

                // 合并activity

                Dictionary<string, XmlNode> FYActivities = new Dictionary<string, XmlNode>();
                XmlNodeList activities = FYapplication.SelectNodes("activity");
                foreach (XmlNode act in activities)
                {
                    XmlNode per = act.Attributes.GetNamedItem("android:name");
                    FYActivities.Add(per.Value, act);
                }
                activities = application.SelectNodes("activity");

                var listAct = new List<string>() {
                    "com.feiyu.sandbox.platform.activity.FYSPUserCenterActivity",
                    "com.feiyu.sandbox.platform.activity.FYSPRealNameActivity",
                    "com.stars.service.activity.FYServiceActivity",
                    "com.stars.advirtual.activity.FYVARewardActivity",
                    "com.stars.combine.activity.FYCombineUseCenterActivity"
                };
                var setAct = new HashSet<string>();
                foreach (var act in listAct)
                {
                    if (!FYActivities.ContainsKey(act))
                    {
                        setAct.Add(act);
                    }
                }
                var delNodes = new List<XmlNode>();

                Dictionary<XmlNode, XmlNode> RPActivities = new Dictionary<XmlNode, XmlNode>();
                refNode = null;
                foreach (XmlNode act in activities)
                {
                    XmlNode per = act.Attributes.GetNamedItem("android:name");
                    if (FYActivities.ContainsKey(per.Value))
                    {
                        RPActivities.Add(FYActivities[per.Value], act);
                        FYActivities.Remove(per.Value);
                    }
                    else
                    {
                        XmlNode intent = act.SelectSingleNode("intent-filter");
                        if (intent != null)
                        {
                            XmlNode _action = intent.SelectSingleNode("action");
                            if (_action != null)
                            {
                                XmlNode an = _action.Attributes.GetNamedItem("android:name");
                                if (an.Value == "android.intent.action.MAIN")
                                    throw new ArgumentException($"存在其他启动的Activity：{per.Value},与目标配置com.stars.bridge.FYSDKActivity不符,只允许存在一个<action android:name=\"android.intent.action.MAIN\" />, 请使用以下方案调整：\r\n\r\n方案一:在Assets/Plugins目录下新建目录(XXXConfig/Editor/)，并创建编辑器脚本，参考{nameof(ConfigWindow)}.{nameof(RegisterConfig)}方法，新建AndroidBuildConfig.json并填写: {{\"FYSDKActivity\":{{\"superClass\":\"{per.Value}\"}}}}，并注册自定义的SDK配置路径到事件{nameof(ConfigWindow)}.{nameof(AddAndroidConfigPathsAct)}中,最后将AndroidManifest文件的{per.Value}修改为com.stars.bridge.FYSDKActivity\r\n\r\n方案二(每次星云配置后会还原修改，不推荐)：请考虑使用继承方式(com.stars.bridge.FYSDKActivity extends {per.Value})或者将{per.Value}代码融合到com.stars.bridge.FYSDKActivity中,最后将AndroidManifest文件的{per.Value}修改为com.stars.bridge.FYSDKActivity\r\n\r\n");
                            }
                        }
                    }

                    if (setAct.Contains(per.Value))
                    {
                        delNodes.Add(act);
                    }
                    else
                    {
                        refNode = act;
                    }

                }

                foreach (var delNode in delNodes)
                {
                    tips.Add($"<color=red>删除:<activity android:name=\"{delNode.Attributes.GetNamedItem("android:name").Value}\" ... ... /> </color>\r\n");
                    application.RemoveChild(delNode);
                }

                foreach (KeyValuePair<XmlNode, XmlNode> kv in RPActivities)
                {
                    XmlNode node = application.OwnerDocument.ImportNode(kv.Key, true);
                    if (IsSatisfyNode(kv.Value, node)) continue;
                    refNode = node;
                    application.ReplaceChild(node, kv.Value);
                    tips.Add($"<color=blue>替换:<activity android:name=\"{node.Attributes.GetNamedItem("android:name").Value}\" ... ... /> </color>\r\n");
                }
                foreach (XmlNode act in FYActivities.Values)
                {
                    XmlNode node_new = manifest.OwnerDocument.ImportNode(act, true);
                    if (refNode == null)
                        application.AppendChild(node_new);
                    else
                        application.InsertAfter(node_new, refNode);
                    refNode = node_new;
                    tips.Add($"<color=green>新增:<activity android:name=\"{node_new.Attributes.GetNamedItem("android:name").Value}\" ... ... /> </color>\r\n");
                }

                // 合并service

                Dictionary<string, XmlNode> fyServicesDict = new Dictionary<string, XmlNode>();
                XmlNodeList fyServicesXNL = FYapplication.SelectNodes("service");
                foreach (XmlNode item in fyServicesXNL)
                {
                    XmlNode per = item.Attributes.GetNamedItem("android:name");
                    fyServicesDict.Add(per.Value, item);
                }
                XmlNodeList servicesXNL = application.SelectNodes("service");
                foreach (XmlNode item in servicesXNL)
                {
                    XmlNode per = item.Attributes.GetNamedItem("android:name");
                    if (fyServicesDict.ContainsKey(per.Value))
                    {
                        fyServicesDict.Remove(per.Value);
                    }

                }
                foreach (XmlNode item in fyServicesDict.Values)
                {
                    XmlNode node_new = manifest.OwnerDocument.ImportNode(item, true);
                    application.AppendChild(node_new);
                    tips.Add($"<color=green>新增:<service android:name=\"{node_new.Attributes.GetNamedItem("android:name").Value}\" ... ... /> </color>\r\n");
                }

                // 合并provider

                Dictionary<string, XmlNode> fyProviderDict = new Dictionary<string, XmlNode>();
                XmlNodeList fyProviderXNL = FYapplication.SelectNodes("provider");
                foreach (XmlNode item in fyProviderXNL)
                {
                    XmlNode per = item.Attributes.GetNamedItem("android:name");
                    fyProviderDict.Add(per.Value, item);
                }
                XmlNodeList providerXNL = application.SelectNodes("provider");
                foreach (XmlNode item in providerXNL)
                {
                    XmlNode per = item.Attributes.GetNamedItem("android:name");
                    if (fyProviderDict.ContainsKey(per.Value))
                    {
                        fyProviderDict.Remove(per.Value);
                    }

                }
                foreach (XmlNode item in fyProviderDict.Values)
                {
                    XmlNode node_new = manifest.OwnerDocument.ImportNode(item, true);
                    application.AppendChild(node_new);
                    tips.Add($"<color=green>新增:<provider android:name=\"{node_new.Attributes.GetNamedItem("android:name").Value}\" ... ... /> </color>\r\n");
                }
            }
            return androidxml;
        }

        static void SetAttr(XmlElement parent, string name, XmlElement FYParent)
        {
            XmlNode attr = parent.Attributes.GetNamedItem(name);
            string val = FYParent.Attributes.GetNamedItem(name).Value;
            if (attr != null)
                attr.Value = val;
            else
                parent.SetAttribute(name, val);
        }
        static void SetAttr(XmlNode parent, string name, XmlNode FYParent)
        {
            XmlNode attr = parent.Attributes.GetNamedItem(name);
            string val = FYParent.Attributes.GetNamedItem(name).Value;
            if (attr != null)
            {
                attr.Value = val;
            }
            else
            {
                XmlNode node_new = FYParent.Attributes.GetNamedItem(name);
                node_new = parent.OwnerDocument.ImportNode(node_new, true);
                parent.Attributes.Append((XmlAttribute)node_new);
            }
        }
        static void SetNode(XmlElement parent, string nodeName, XmlElement FYParent)
        {
            XmlNode node = parent.SelectSingleNode(nodeName);
            XmlNode node_new = FYParent.SelectSingleNode(nodeName);
            node_new = parent.OwnerDocument.ImportNode(node_new, true);
            if (node != null)
                parent.ReplaceChild(node_new, node);
            else
                parent.AppendChild(node_new);
        }

        static bool IsSatisfyAttrs(XmlNode cur, XmlNode refer)
        {
            var referAttrs = refer.Attributes.Cast<XmlAttribute>();
            var curAttrs = cur.Attributes.Cast<XmlAttribute>();
            return referAttrs.Where((attr) => curAttrs.Where(ca => ca.Name == attr.Name && ca.Value == attr.Value).Count() > 0).Count() == referAttrs.Count();
        }
        static bool IsSatisfyNode(XmlNode cur, XmlNode refer)
        {
            if (cur.Name != refer.Name) return false;
            if (!IsSatisfyAttrs(cur, refer)) return false;
            if (refer.ChildNodes.Count == 0) return true;
            var referNodes = refer.ChildNodes.Cast<XmlNode>();
            var curNodes = cur.ChildNodes.Cast<XmlNode>();
            foreach (var node in referNodes)
            {
                int cnsC = curNodes.Where(cn => IsSatisfyNode(cn, node)).Count();
                if (cnsC == 0) return false;
            }
            return true;
        }
        #endregion
        internal static void Warning(string fileName)
        {
            Debug.LogWarning($"FYINFO: 目标文件{fileName}配置存在差异，可能导致接入的SDK无法正常使用，{HelpInfo}");
        }
        internal static void Error(string fileName)
        {
            throw new FileNotFoundException($"FYINFO: 需要的目标文件{fileName}不存在，{HelpInfo}");
        }
    }
}